@@ -577,7 +577,6 @@ uint16_t exfat_calc_name_hash(struct exfat *exfat,
 
 	for (i = 0; i < len; i++) {
 		ch = exfat->upcase_table[le16_to_cpu(name[i])];
-		ch = cpu_to_le16(ch);
 
 		/* use += to avoid promotion to int; UBSan complaints about signed overflow */
 		chksum = (chksum << 15) | (chksum >> 1);
@@ -692,7 +691,7 @@ int exfat_update_file_dentry_set(struct exfat *exfat,
 
 		dset[1].dentry.stream.name_len = (__u8)name_len;
 		dset[1].dentry.stream.name_hash =
-			exfat_calc_name_hash(exfat, utf16_name, name_len);
+			cpu_to_le16(exfat_calc_name_hash(exfat, utf16_name, name_len));
 
 		for (i = 2; i < dcount; i++) {
 			dset[i].type = EXFAT_NAME;
@@ -712,15 +711,9 @@ int exfat_update_file_dentry_set(struct exfat *exfat,
 	return 0;
 }
 
-static int find_free_cluster(struct exfat *exfat,
-			     clus_t start, clus_t *new_clu)
+static int __find_free_cluster(struct exfat *exfat, clus_t *new_clu,
+		clus_t start, clus_t end)
 {
-	clus_t end = le32_to_cpu(exfat->bs->bsx.clu_count) +
-		EXFAT_FIRST_CLUSTER;
-
-	if (!exfat_heap_clus(exfat, start))
-		return -EINVAL;
-
 	while (start < end) {
 		if (exfat_bitmap_find_zero(exfat, exfat->alloc_bitmap,
 					   start, new_clu))
@@ -730,20 +723,61 @@ static int find_free_cluster(struct exfat *exfat,
 		start = *new_clu + 1;
 	}
 
-	end = start;
-	start = EXFAT_FIRST_CLUSTER;
-	while (start < end) {
-		if (exfat_bitmap_find_zero(exfat, exfat->alloc_bitmap,
-					   start, new_clu))
-			goto out_nospc;
-		if (!exfat_bitmap_get(exfat->disk_bitmap, *new_clu))
-			return 0;
-		start = *new_clu + 1;
+	*new_clu = EXFAT_EOF_CLUSTER;
+
+	return -ENOSPC;
+}
+
+static int find_free_cluster(struct exfat *exfat,
+			     clus_t start, clus_t *new_clu)
+{
+	clus_t end = le32_to_cpu(exfat->bs->bsx.clu_count) +
+		EXFAT_FIRST_CLUSTER;
+
+	if (!exfat_heap_clus(exfat, start))
+		return -EINVAL;
+
+	if (__find_free_cluster(exfat, new_clu, start, end) == 0)
+		return 0;
+
+	return __find_free_cluster(exfat, new_clu, EXFAT_FIRST_CLUSTER, start);
+}
+
+/* Find multiple contiguous free clusters */
+int exfat_find_free_cluster(struct exfat *exfat, int clu_count,
+			    clus_t *new_clu)
+{
+	int ret, i;
+	clus_t clu;
+	clus_t start = EXFAT_FIRST_CLUSTER;
+	clus_t end = le32_to_cpu(exfat->bs->bsx.clu_count) +
+		EXFAT_FIRST_CLUSTER;
+
+find_again:
+	ret = __find_free_cluster(exfat, &clu, start, end);
+	if (ret < 0)
+		return ret;
+
+	*new_clu = clu;
+	start = clu + 1;
+
+	for (i = 1; i < clu_count && start < end; i++) {
+		ret = __find_free_cluster(exfat, &clu, start, end);
+		if (ret < 0)
+			return ret;
+
+		if (clu != start) {
+			start = clu;
+			goto find_again;
+		}
+
+		start = clu + 1;
 	}
 
-out_nospc:
-	*new_clu = EXFAT_EOF_CLUSTER;
-	return -ENOSPC;
+	if (start >= end)
+		return -ENOSPC;
+
+	return 0;
 }
 
 static int exfat_map_cluster(struct exfat *exfat, struct exfat_inode *inode,
@@ -829,8 +863,8 @@ static int exfat_write_dentry_set(struct exfat *exfat,
 	return 0;
 }
 
-static int exfat_alloc_cluster(struct exfat *exfat, struct exfat_inode *inode,
-			       clus_t *new_clu)
+int exfat_alloc_cluster(struct exfat *exfat, struct exfat_inode *inode,
+		clus_t *new_clu)
 {
 	clus_t last_clu;
 	int err;
